@@ -74,7 +74,7 @@
 
 #include "gsttypefindelement.h"
 #include "gstcoreelementselements.h"
-#include "gst/gst-i18n-lib.h"
+#include <glib/gi18n-lib.h>
 #include "gst/base/gsttypefindhelper.h"
 
 #include <gst/gsttypefind.h>
@@ -490,7 +490,7 @@ gst_type_find_element_seek (GstTypeFindElement * typefind, GstEvent * event)
   gst_segment_do_seek (&seeksegment, rate, format, flags,
       start_type, start, stop_type, stop, NULL);
 
-  flush = ! !(flags & GST_SEEK_FLAG_FLUSH);
+  flush = !!(flags & GST_SEEK_FLAG_FLUSH);
 
   GST_DEBUG_OBJECT (typefind, "New segment %" GST_SEGMENT_FORMAT, &seeksegment);
 
@@ -803,6 +803,8 @@ gst_type_find_get_extension (GstTypeFindElement * typefind, GstPad * pad)
   gchar *uri, *result, *path, *base_path, *find;
   GstUri *gst_uri;
 
+  base_path = NULL;
+
   query = gst_query_new_uri ();
 
   /* try getting the caps with an uri query and from the extension */
@@ -813,6 +815,11 @@ gst_type_find_get_extension (GstTypeFindElement * typefind, GstPad * pad)
   if (uri == NULL)
     goto no_uri;
 
+  /* data URIs paths are opaque and do not semantically represent a
+     filesystem-like resource path, so skip URI parsing for this case. */
+  if (g_str_has_prefix (uri, "data:"))
+    goto no_extension;
+
   GST_DEBUG_OBJECT (typefind, "finding extension of %s", uri);
 
   gst_uri = gst_uri_from_string (uri);
